@@ -25,9 +25,4 @@
 # **************************************************************************
 
 from .protocol_motioncorr import ProtMotionCorr
-
-try:
-    from .protocol_ts_motioncor import ProtTsMotionCorr
-except ImportError as e:
-    if "'tomo'" not in str(e):
-        raise e
+from .protocol_ts_motioncor import ProtTsMotionCorr
