@@ -37,15 +37,18 @@ from threading import Thread
 
 import pyworkflow.protocol.constants as cons
 import pyworkflow.protocol.params as params
+import pyworkflow.object as pwobj
 import pyworkflow.utils as pwutils
-from pwem.objects import Image, Float
-from pwem.protocols import ProtAlignMovies
 from pyworkflow.gui.plotter import Plotter
 from pyworkflow.protocol import STEPS_PARALLEL
+from pwem.emlib.image import ImageHandler
+from pwem.objects import Image, Float
+from pwem.protocols import ProtAlignMovies
 
 from .. import Plugin
 from ..constants import *
 from ..convert import *
+from relion.convert.convert31 import OpticsGroups
 
 
 class ProtMotionCorr(ProtAlignMovies):
@@ -56,11 +59,11 @@ class ProtMotionCorr(ProtAlignMovies):
     """
 
     _label = 'movie alignment'
-    CONVERT_TO_MRC = 'mrc'
 
-    def __init__(self, **args):
-        ProtAlignMovies.__init__(self, **args)
+    def __init__(self, **kwargs):
+        ProtAlignMovies.__init__(self, **kwargs)
         self.stepsExecutionMode = STEPS_PARALLEL
+        self.isEER = False
 
     def versionGE(self, version):
         """ Return True if current version of motioncor2 is greater
@@ -78,7 +81,7 @@ class ProtMotionCorr(ProtAlignMovies):
     def _getConvertExtension(self, filename):
         """ Check whether it is needed to convert to .mrc or not """
         ext = pwutils.getExt(filename).lower()
-        return None if ext in ['.mrc', '.mrcs', '.tiff', '.tif'] else 'mrc'
+        return None if ext in ['.mrc', '.mrcs', '.tiff', '.tif', '.eer'] else 'mrc'
 
     # -------------------------- DEFINE param functions -----------------------
     def _defineAlignmentParams(self, form):
@@ -101,7 +104,8 @@ class ProtMotionCorr(ProtAlignMovies):
                              help='Frames range to ALIGN and SUM on each movie. The '
                                   'first frame is 1. If you set 0 in the final '
                                   'frame to align, it means that you will '
-                                  'align until the last frame of the movie.')
+                                  'align until the last frame of the movie. '
+                                  'When using EER, this option is IGNORED!')
         line.addParam('alignFrame0', params.IntParam, default=1,
                       label='from')
         line.addParam('alignFrameN', params.IntParam, default=0,
@@ -236,6 +240,22 @@ class ProtMotionCorr(ProtAlignMovies):
                            'can be provided as either MRC or TIFF file that has '
                            'MRC mode of 0 or 5 (unsigned 8 bit).')
 
+        form.addSection("EER")
+        form.addParam('EERtext', params.LabelParam,
+                      label="These options are ignored for non-EER movies.")
+        form.addParam('eerGroup', params.IntParam, default=32,
+                      label='EER fractionation',
+                      help="The number of hardware frames to group into one "
+                           "fraction. This option is relevant only for Falcon4 "
+                           "movies in the EER format. Falcon 4 operates at "
+                           "248 frames/s.\nFractionate such that each fraction "
+                           "has about 0.5 to 1.25 e/A2.")
+        form.addParam('eerSampling', params.EnumParam, default=0,
+                      choices=['1x', '2x', '4x'],
+                      display=params.EnumParam.DISPLAY_HLIST,
+                      label='EER upsampling',
+                      help="EER upsampling (1x = 4K, 2x = 8K, 3x=16K)")
+
         form.addSection(label="Mag. correction")
         form.addParam('doMagCor', params.BooleanParam, default=False,
                       label='Correct anisotropic magnification?',
@@ -256,19 +276,40 @@ class ProtMotionCorr(ProtAlignMovies):
         form.addParallelSection(threads=1, mpi=1)
 
     # --------------------------- STEPS functions -----------------------------
+    def _convertInputStep(self):
+        inputMovies = self.getInputMovies()
+        # parse EER gain file before its conversion to mrc
+        if self.isEER and inputMovies.getGain():
+            defects = parseEERDefects(inputMovies.getGain())
+            if defects:
+                with open(self._getExtraPath("defects_eer.txt"), "w") as f:
+                    for d in defects:
+                        f.write(" ".join(str(i) for i in d) + "\n")
+
+        ProtAlignMovies._convertInputStep(self)
+        if self.isEER:
+            # write FmIntFile
+            _, numbOfFrames, _ = inputMovies.getFramesRange()
+            if self.doApplyDoseFilter:
+                _, dose = self._getCorrectedDose(inputMovies)
+            else:
+                dose = 0.0
+            with open(self._getExtraPath("FmIntFile.txt"), "w") as f:
+                f.write("%d %d %f" % (numbOfFrames, self.eerGroup.get(), dose))
+
     def _processMovie(self, movie):
         inputMovies = self.getInputMovies()
         movieFolder = self._getOutputMovieFolder(movie)
         outputMicFn = self._getOutputMicName(movie)
-        a0, aN = self._getRange(movie, 'align')
         program = self._getProgram()
         logFileBase = self._getMovieRoot(movie) + "_"
-        numbOfFrames = self._getNumberOfFrames(movie)
+        frame0, frameN = self._getFrameRange()
+        _, numbOfFrames, _ = inputMovies.getFramesRange()
 
         argsDict = self._getArgs()
         argsDict.update({'-OutMrc': '"%s"' % outputMicFn,
-                         '-Throw': '%d' % a0,
-                         '-Trunc': '%d' % (abs(aN - numbOfFrames + 1)),
+                         '-Throw': '%d' % 0 if self.isEER else (frame0 - 1),
+                         '-Trunc': '%d' % 0 if self.isEER else (numbOfFrames - frameN),
                          '-LogFile': logFileBase,
                          })
 
@@ -277,6 +318,8 @@ class ProtMotionCorr(ProtAlignMovies):
             args = ' -InMrc "%s" ' % movie.getBaseName()
         elif ext in ['.tif', '.tiff']:
             args = ' -InTiff "%s" ' % movie.getBaseName()
+        elif ext == '.eer':
+            args = ' -InEer "%s" ' % movie.getBaseName()
         else:
             raise Exception("Unsupported format: %s" % ext)
 
@@ -294,7 +337,6 @@ class ProtMotionCorr(ProtAlignMovies):
                 # we need to move shifts log to extra dir before parsing
                 try:
                     self._saveAlignmentPlots(movie, inputMovies.getSamplingRate())
-
                     outMicFn = self._getExtraPath(self._getMicFn(movie))
 
                     if self.doComputePSD:
@@ -305,7 +347,7 @@ class ProtMotionCorr(ProtAlignMovies):
                             outMicFn, outputFn=self._getOutputMicThumbnail(movie))
                 except:
                     self.error("ERROR: Extra work "
-                               "(i.e plots, PSD, thumbnail has failed for %s\n"
+                               "(i.e plots, PSD, thumbnail) has failed for %s\n"
                                % movie.getFileName())
 
             if self._useWorkerThread():
@@ -358,17 +400,46 @@ class ProtMotionCorr(ProtAlignMovies):
         return methods
 
     def _validate(self):
-        # Check base validation before the specific ones
-        errors = ProtAlignMovies._validate(self)
+        errors = []
+        inputMovies = self.inputMovies.get()
 
-        if self.doApplyDoseFilter and self.inputMovies.get():
-            inputMovies = self.inputMovies.get()
-            doseFrame = inputMovies.getAcquisition().getDosePerFrame()
+        # check if the first movie exists
+        firstMovie = self.inputMovies.get().getFirstItem()
+        if not os.path.exists(firstMovie.getFileName()):
+            errors.append("The input movie files do not exist!!! "
+                          "Since usually input movie files are symbolic links, "
+                          "please check that links are not broken if you "
+                          "moved the project folder. ")
 
-            if doseFrame < 0.00001 or doseFrame is None:
-                errors.append('Dose per frame for input movies is 0 or not '
-                              'set. You cannot apply dose filter.')
+        # check frames range
+        _, lastFrame, _ = inputMovies.getFramesRange()
+        self.isEER = pwutils.getExt(firstMovie.getFileName()) == ".eer"
+        if self.isEER:
+            if not self.versionGE('1.4.0'):
+                errors.append("EER is only supported for motioncor2 v1.4.0 or newer.")
+            if self.alignFrame0.get() != 1 or self.alignFrameN.get() not in [0, lastFrame]:
+                errors.append("For EER data please set frame range from 1 to 0 (or 1 to %d)." % lastFrame)
 
+        msg = "Frames range must be within %d - %d" % (1, lastFrame)
+        if self.alignFrameN.get() == 0:
+            self.alignFrameN.set(lastFrame)
+
+        if not (1 <= self.alignFrame0 < lastFrame):
+            errors.append(msg)
+        elif not (self.alignFrameN <= lastFrame):
+            errors.append(msg)
+        elif not (self.alignFrame0 < self.alignFrameN):
+            errors.append(msg)
+
+        # check dose for DW
+        acq = inputMovies.getAcquisition()
+        if self.doApplyDoseFilter:
+            dose = acq.getDosePerFrame()
+            if dose is None or dose < 0.00001:
+                errors.append("Input movies do not contain the dose per frame, "
+                              "dose-weighting can not be performed.")
+
+        # check eman2 plugin
         if self.doComputeMicThumbnail or self.doComputePSD:
             try:
                 from pwem import Domain
@@ -377,6 +448,19 @@ class ProtMotionCorr(ProtAlignMovies):
                 errors.append("EMAN2 plugin not found!\nComputing thumbnails "
                               "or PSD requires EMAN2 plugin and binaries installed.")
 
+        # check gain dimensions and extension
+        if inputMovies.getGain():
+            ih = ImageHandler()
+            gain = inputMovies.getGain()
+            gainx, gainy, _, _ = ih.getDimensions(gain)
+            movie = firstMovie.getFileName()
+            imgx, imgy, _, _ = ih.getDimensions(movie)
+
+            if sorted([gainx, gainy]) != sorted([imgx, imgy]):
+                errors.append("Gain image dimensions (%d x %d) "
+                              "do not match the movies (%d x %d)!" %
+                              (gainx, gainy, imgx, imgy))
+
         return errors
 
     # --------------------------- UTILS functions -----------------------------
@@ -405,18 +489,24 @@ class ProtMotionCorr(ProtAlignMovies):
                     '-kV': inputMovies.getAcquisition().getVoltage(),
                     '-OutStack': 1 if self.doSaveMovie else 0,
                     '-Gpu': '%(GPU)s',
-                    '-SumRange': "0.0 0.0",  # switch off writing out DWS
+                    '-SumRange': "0.0 0.0",  # switch off writing out DWS,
+                    #'-FmRef': 0
                     }
 
-        if self.doApplyDoseFilter:
+        if self.isEER:
+            argsDict['-EerSampling'] = self.eerSampling.get() + 1
+            argsDict['-FmIntFile'] = "../../extra/FmIntFile.txt"
+        elif self.doApplyDoseFilter:
             preExp, dose = self._getCorrectedDose(inputMovies)
             argsDict.update({'-FmDose': dose,
                              '-InitDose': preExp})
 
         if self.defectFile.get():
             argsDict['-DefectFile'] = "%s" % self.defectFile.get()
-        if self.defectMap.get():
+        elif self.defectMap.get():
             argsDict['-DefectMap'] = "%s" % self.defectMap.get()
+        elif os.path.exists(self._getExtraPath("defects_eer.txt")):
+            argsDict['-DefectFile'] = "../../extra/defects_eer.txt"
 
         patchOverlap = self.getAttributeValue('patchOverlap')
         if patchOverlap:  # 0 or None is False
@@ -437,6 +527,12 @@ class ProtMotionCorr(ProtAlignMovies):
 
         return argsDict
 
+    def _getBinFactor(self):
+        if not self.isEER:
+            return self.binFactor.get()
+        else:
+            return self.binFactor.get() / (self.eerSampling.get() + 1)
+
     def _getVersion(self):
         return Plugin.getActiveVersion()
 
@@ -512,14 +608,13 @@ class ProtMotionCorr(ProtAlignMovies):
     def _saveAlignmentPlots(self, movie, pixSize):
         """ Compute alignment shift plots and save to file as png images. """
         shiftsX, shiftsY = self._getMovieShifts(movie)
-        first, _ = self._getFrameRange(movie.getNumberOfFrames(), 'align')
+        first, _ = self._getFrameRange()
         plotter = createGlobalAlignmentPlot(shiftsX, shiftsY, first, pixSize)
         plotter.savefig(self._getPlotGlobal(movie))
         plotter.close()
 
     def _moveOutput(self, movie):
         """ Move output from tmp to extra folder. """
-
         def _moveToExtra(fn):
             """ Move file from movies tmp folder to extra """
             pwutils.moveFile(self._getCwdPath(movie, fn),
@@ -538,31 +633,22 @@ class ProtMotionCorr(ProtAlignMovies):
                                           '_aligned_mic_Stk.mrc')
             pwutils.moveFile(movieFn, outputMovieFn)
 
-    def _getRange(self, movie, prefix):
-        n = self._getNumberOfFrames(movie)
-        iniFrame, _, indxFrame = movie.getFramesRange()
-        first, last = self._getFrameRange(n, prefix)
+    def _updateOutputSet(self, outputName, outputSet,
+                         state=pwobj.Set.STREAM_OPEN):
+        """ Redefine this method to set EER attrs. """
+        first = getattr(self, '_firstUpdate', True)
 
-        if iniFrame != indxFrame:
-            first -= iniFrame
-            last -= iniFrame
-        else:
-            first -= 1
-            last -= 1
+        if first and outputName == 'outputMovies':
+            og = OpticsGroups.fromImages(outputSet)
+            if self.isEER:
+                og.updateAll(rlnEERGrouping=self.eerGroup.get(),
+                             rlnEERUpsampling=self.eerSampling.get() + 1)
+            og.updateAll(rlnMicrographStartFrame=self.alignFrame0.get())
+            og.toImages(outputSet)
 
-        return first, last
-
-    def _getNumberOfFrames(self, movie):
-        _, lstFrame, _ = movie.getFramesRange()
-
-        if movie.hasAlignment():
-            _, lastFrmAligned = movie.getAlignment().getRange()
-            if lastFrmAligned != lstFrame:
-                return lastFrmAligned
-            else:
-                return movie.getNumberOfFrames()
-        else:
-            return movie.getNumberOfFrames()
+        ProtAlignMovies._updateOutputSet(self, outputName, outputSet,
+                                         state=state)
+        self._firstUpdate = False
 
     def _createOutputMicrographs(self):
         createWeighted = self._createOutputWeightedMicrographs()
@@ -590,9 +676,12 @@ class ProtMotionCorr(ProtAlignMovies):
     def calcFrameMotion(self, movie):
         # based on relion 3.1 motioncorr_runner.cpp
         shiftsX, shiftsY = self._getMovieShifts(movie)
-        a0, aN = self._getRange(movie, "align")
+        a0, aN = self._getFrameRange()
         nframes = aN - a0 + 1
         preExp, dose = self._getCorrectedDose(self.getInputMovies())
+        # when using EER, the hardware frames are grouped
+        if self.isEER:
+            dose *= self.eerGroup.get()
         cutoff = (4 - preExp) // dose  # early is <= 4e/A^2
         total, early, late = 0., 0., 0.
         x, y, xOld, yOld = 0., 0., 0., 0.
@@ -613,6 +702,13 @@ class ProtMotionCorr(ProtAlignMovies):
             self.error("Expected %d frames, found less. Check movie %s !" % (
                 nframes, movie.getFileName()))
 
+    def _getFrameRange(self, n=None, prefix=None):
+        # Reimplement this method
+        if self.isEER:
+            return self.alignFrame0.get(), self.alignFrameN.get() // self.eerGroup.get()
+        else:
+            return self.alignFrame0.get(), self.alignFrameN.get()
+
 
 def createGlobalAlignmentPlot(meanX, meanY, first, pixSize):
     """ Create a plotter with the shift per frame. """
